@@ -1,38 +1,27 @@
-from datetime import datetime
-
 import django
 from django import forms
 from django.conf import settings
-from django.conf.urls import url
 from django.contrib import admin, messages
 from django.contrib.admin.models import ADDITION, CHANGE, DELETION, LogEntry
 from django.contrib.auth import get_permission_codename
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import PermissionDenied
 from django.http import HttpResponse, HttpResponseRedirect
+from django.template.defaultfilters import escape
 from django.template.response import TemplateResponse
-from django.urls import reverse
+from django.urls import path, reverse
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_str
 from django.utils.module_loading import import_string
 from django.utils.translation import gettext_lazy as _
 from django.views.decorators.http import require_POST
 
-from .formats.base_formats import DEFAULT_FORMATS
 from .forms import ConfirmImportForm, ExportForm, ImportForm, export_action_form_factory
-from .resources import modelresource_factory
+from .mixins import BaseExportMixin, BaseImportMixin
 from .results import RowResult
 from .signals import post_export, post_import
 from .tmp_storages import TempFolderStorage
 
-SKIP_ADMIN_LOG = getattr(settings, 'IMPORT_EXPORT_SKIP_ADMIN_LOG', False)
-TMP_STORAGE_CLASS = getattr(settings, 'IMPORT_EXPORT_TMP_STORAGE_CLASS',
-                            TempFolderStorage)
-
-
-if isinstance(TMP_STORAGE_CLASS, str):
-    TMP_STORAGE_CLASS = import_string(TMP_STORAGE_CLASS)
-
 
 class ImportExportMixinBase:
     def get_model_info(self):
@@ -40,22 +29,18 @@ class ImportExportMixinBase:
         return (app_label, self.model._meta.model_name)
 
 
-class ImportMixin(ImportExportMixinBase):
+class ImportMixin(BaseImportMixin, ImportExportMixinBase):
     """
     Import mixin.
 
     This is intended to be mixed with django.contrib.admin.ModelAdmin
-    https://docs.djangoproject.com/en/2.1/ref/contrib/admin/#modeladmin-objects
+    https://docs.djangoproject.com/en/dev/ref/contrib/admin/
     """
 
     #: template for change_list view
     change_list_template = 'admin/import_export/change_list_import.html'
     #: template for import view
     import_template_name = 'admin/import_export/import.html'
-    #: resource class
-    resource_class = None
-    #: available import formats
-    formats = DEFAULT_FORMATS
     #: import data encoding
     from_encoding = "utf-8"
     skip_admin_log = None
@@ -64,15 +49,21 @@ class ImportMixin(ImportExportMixinBase):
 
     def get_skip_admin_log(self):
         if self.skip_admin_log is None:
-            return SKIP_ADMIN_LOG
+            return getattr(settings, 'IMPORT_EXPORT_SKIP_ADMIN_LOG', False)
         else:
             return self.skip_admin_log
 
     def get_tmp_storage_class(self):
         if self.tmp_storage_class is None:
-            return TMP_STORAGE_CLASS
+            tmp_storage_class = getattr(
+                settings, 'IMPORT_EXPORT_TMP_STORAGE_CLASS', TempFolderStorage,
+            )
         else:
-            return self.tmp_storage_class
+            tmp_storage_class = self.tmp_storage_class
+
+        if isinstance(tmp_storage_class, str):
+            tmp_storage_class = import_string(tmp_storage_class)
+        return tmp_storage_class
 
     def has_import_permission(self, request):
         """
@@ -90,41 +81,15 @@ class ImportMixin(ImportExportMixinBase):
         urls = super().get_urls()
         info = self.get_model_info()
         my_urls = [
-            url(r'^process_import/$',
+            path('process_import/',
                 self.admin_site.admin_view(self.process_import),
                 name='%s_%s_process_import' % info),
-            url(r'^import/$',
+            path('import/',
                 self.admin_site.admin_view(self.import_action),
                 name='%s_%s_import' % info),
         ]
         return my_urls + urls
 
-    def get_resource_kwargs(self, request, *args, **kwargs):
-        return {}
-
-    def get_import_resource_kwargs(self, request, *args, **kwargs):
-        """Prepares/returns kwargs used when initializing Resource"""
-        return self.get_resource_kwargs(request, *args, **kwargs)
-
-    def get_resource_class(self):
-        """Returns ResourceClass"""
-        if not self.resource_class:
-            return modelresource_factory(self.model)
-        else:
-            return self.resource_class
-
-    def get_import_resource_class(self):
-        """
-        Returns ResourceClass to use for import.
-        """
-        return self.get_resource_class()
-
-    def get_import_formats(self):
-        """
-        Returns available import formats.
-        """
-        return [f for f in self.formats if f().can_import()]
-
     @method_decorator(require_POST)
     def process_import(self, request, *args, **kwargs):
         """
@@ -295,9 +260,10 @@ class ImportMixin(ImportExportMixinBase):
                     data = force_str(data, self.from_encoding)
                 dataset = input_format.create_dataset(data)
             except UnicodeDecodeError as e:
-                return HttpResponse(_(u"<h1>Imported file has a wrong encoding: %s</h1>" % e))
+                return HttpResponse(_(u"<h1>Imported file has a wrong encoding: %s</h1>" % escape(e)))
             except Exception as e:
-                return HttpResponse(_(u"<h1>%s encountered while trying to read file: %s</h1>" % (type(e).__name__, import_file.name)))
+                return HttpResponse(_(u"<h1>%s encountered while trying to read file: %s</h1>" % (
+                    escape(type(e).__name__), escape(import_file.name))))
 
             # prepare kwargs for import data, if needed
             res_kwargs = self.get_import_resource_kwargs(request, form=form, *args, **kwargs)
@@ -331,6 +297,7 @@ class ImportMixin(ImportExportMixinBase):
         context['title'] = _("Import")
         context['form'] = form
         context['opts'] = self.model._meta
+        context['media'] = self.media + form.media
         context['fields'] = [f.column_name for f in resource.get_user_visible_fields()]
 
         request.current_app = self.admin_site.name
@@ -344,28 +311,24 @@ class ImportMixin(ImportExportMixinBase):
         return super().changelist_view(request, extra_context)
 
 
-class ExportMixin(ImportExportMixinBase):
+class ExportMixin(BaseExportMixin, ImportExportMixinBase):
     """
     Export mixin.
 
     This is intended to be mixed with django.contrib.admin.ModelAdmin
-    https://docs.djangoproject.com/en/2.1/ref/contrib/admin/#modeladmin-objects
+    https://docs.djangoproject.com/en/dev/ref/contrib/admin/
     """
-    #: resource class
-    resource_class = None
     #: template for change_list view
     change_list_template = 'admin/import_export/change_list_export.html'
     #: template for export view
     export_template_name = 'admin/import_export/export.html'
-    #: available export formats
-    formats = DEFAULT_FORMATS
     #: export data encoding
-    to_encoding = "utf-8"
+    to_encoding = None
 
     def get_urls(self):
         urls = super().get_urls()
         my_urls = [
-            url(r'^export/$',
+            path('export/',
                 self.admin_site.admin_view(self.export_action),
                 name='%s_%s_export' % self.get_model_info()),
         ]
@@ -383,37 +346,6 @@ class ExportMixin(ImportExportMixinBase):
         codename = get_permission_codename(EXPORT_PERMISSION_CODE, opts)
         return request.user.has_perm("%s.%s" % (opts.app_label, codename))
 
-    def get_resource_kwargs(self, request, *args, **kwargs):
-        return {}
-
-    def get_export_resource_kwargs(self, request, *args, **kwargs):
-        return self.get_resource_kwargs(request, *args, **kwargs)
-
-    def get_resource_class(self):
-        if not self.resource_class:
-            return modelresource_factory(self.model)
-        else:
-            return self.resource_class
-
-    def get_export_resource_class(self):
-        """
-        Returns ResourceClass to use for export.
-        """
-        return self.get_resource_class()
-
-    def get_export_formats(self):
-        """
-        Returns available export formats.
-        """
-        return [f for f in self.formats if f().can_export()]
-
-    def get_export_filename(self, request, queryset, file_format):
-        date_str = datetime.now().strftime('%Y-%m-%d')
-        filename = "%s-%s.%s" % (self.model.__name__,
-                                 date_str,
-                                 file_format.get_extension())
-        return filename
-
     def get_export_queryset(self, request):
         """
         Returns export queryset.
@@ -442,8 +374,9 @@ class ExportMixin(ImportExportMixinBase):
             'list_editable': self.list_editable,
             'model_admin': self,
         }
-        if django.VERSION >= (2, 1):
-            changelist_kwargs['sortable_by'] = self.sortable_by
+        changelist_kwargs['sortable_by'] = self.sortable_by
+        if django.VERSION >= (4, 0):
+            changelist_kwargs['search_help_text'] = self.search_help_text
         cl = ChangeList(**changelist_kwargs)
 
         return cl.get_queryset(request)
@@ -456,9 +389,11 @@ class ExportMixin(ImportExportMixinBase):
         if not self.has_export_permission(request):
             raise PermissionDenied
 
-        resource_class = self.get_export_resource_class()
-        data = resource_class(**self.get_export_resource_kwargs(request)).export(queryset, *args, **kwargs)
+        data = self.get_data_for_export(request, queryset, *args, **kwargs)
         export_data = file_format.export_data(data)
+        encoding = kwargs.get("encoding")
+        if not file_format.is_binary() and encoding:
+            export_data = export_data.encode(encoding)
         return export_data
 
     def get_export_context_data(self, **kwargs):
@@ -467,19 +402,26 @@ class ExportMixin(ImportExportMixinBase):
     def get_context_data(self, **kwargs):
         return {}
 
+    def get_export_form(self):
+        """
+        Get the form type used to read the export format.
+        """
+        return ExportForm
+
     def export_action(self, request, *args, **kwargs):
         if not self.has_export_permission(request):
             raise PermissionDenied
 
         formats = self.get_export_formats()
-        form = ExportForm(formats, request.POST or None)
+        form_type = self.get_export_form()
+        form = form_type(formats, request.POST or None)
         if form.is_valid():
             file_format = formats[
                 int(form.cleaned_data['file_format'])
             ]()
 
             queryset = self.get_export_queryset(request)
-            export_data = self.get_export_data(file_format, queryset, request=request)
+            export_data = self.get_export_data(file_format, queryset, request=request, encoding=self.to_encoding)
             content_type = file_format.get_content_type()
             response = HttpResponse(export_data, content_type=content_type)
             response['Content-Disposition'] = 'attachment; filename="%s"' % (
@@ -506,6 +448,9 @@ class ExportMixin(ImportExportMixinBase):
         extra_context['has_export_permission'] = self.has_export_permission(request)
         return super().changelist_view(request, extra_context)
 
+    def get_export_filename(self, request, queryset, file_format):
+        return super().get_export_filename(file_format)
+
 
 class ImportExportMixin(ImportMixin, ExportMixin):
     """
@@ -556,17 +501,28 @@ class ExportActionMixin(ExportMixin):
             formats = self.get_export_formats()
             file_format = formats[int(export_format)]()
 
-            export_data = self.get_export_data(file_format, queryset, request=request)
+            export_data = self.get_export_data(file_format, queryset, request=request, encoding=self.to_encoding)
             content_type = file_format.get_content_type()
             response = HttpResponse(export_data, content_type=content_type)
             response['Content-Disposition'] = 'attachment; filename="%s"' % (
                 self.get_export_filename(request, queryset, file_format),
             )
             return response
-    export_admin_action.short_description = _(
-        'Export selected %(verbose_name_plural)s')
 
-    actions = admin.ModelAdmin.actions + [export_admin_action]
+    def get_actions(self, request):
+        """
+        Adds the export action to the list of available actions.
+        """
+
+        actions = super().get_actions(request)
+        actions.update(
+            export_admin_action=(
+                ExportActionMixin.export_admin_action,
+                "export_admin_action",
+                _("Export selected %(verbose_name_plural)s"),
+            )
+        )
+        return actions
 
     @property
     def media(self):
