@@ -85,7 +85,7 @@ class IntegerWidget(NumberWidget):
     def clean(self, value, row=None, *args, **kwargs):
         if self.is_empty(value):
             return None
-        return int(float(value))
+        return int(Decimal(value))
 
 
 class DecimalWidget(NumberWidget):
@@ -212,21 +212,21 @@ class DateTimeWidget(Widget):
         self.formats = formats
 
     def clean(self, value, row=None, *args, **kwargs):
+        dt = None
         if not value:
             return None
         if isinstance(value, datetime):
-            return value
-        for format in self.formats:
-            try:
-                dt = datetime.strptime(value, format)
-                if settings.USE_TZ:
-                    # make datetime timezone aware so we don't compare
-                    # naive datetime to an aware one
-                    dt = timezone.make_aware(dt,
-                                             timezone.get_default_timezone())
-                return dt
-            except (ValueError, TypeError):
-                continue
+            dt = value
+        else:
+            for format_ in self.formats:
+                try:
+                    dt = datetime.strptime(value, format_)
+                except (ValueError, TypeError):
+                    continue
+        if dt:
+            if settings.USE_TZ:
+                dt = timezone.make_aware(dt)
+            return dt
         raise ValueError("Enter a valid date/time.")
 
     def render(self, value, obj=None):
@@ -392,7 +392,7 @@ class ForeignKeyWidget(Widget):
         like so::
 
             class FullNameForeignKeyWidget(ForeignKeyWidget):
-                def get_queryset(self, value, row):
+                def get_queryset(self, value, row, *args, **kwargs):
                     return self.model.objects.filter(
                         first_name__iexact=row["first_name"],
                         last_name__iexact=row["last_name"]
